#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>

#define DEBUG

void USART_init ()
{
	/* setez BAUD rate la 9600 */
	UBRR0H = 0;
    UBRR0L = 103;
	
	/* porneste transmitatorul */
    UCSR0B = (1 << TXEN0) | (1 << RXEN0);

    /* seteaza formatul frame-ului: 8 biti de date, 1 biti de stop */
    UCSR0C &= ~(1 << USBS0);
    UCSR0C |= (2 << UCSZ00);
}

void motor_init()
{
	/* Setez PB3 pentru ENABLE PWM*/
	DDRB |= (1 << PB3);
	
	/* Motor fata/spate */
	DDRC |= (1 << PC3);
	DDRC |= (1 << PC4);
	
	/* Motor stanga/dreapta */
	DDRC |= (1 << PC5);
	DDRC |= (1 << PC6);
	
}

char USART_receive()
{	
	/* asteapta cat timp bufferul e gol */
    while (!(UCSR0A & (1 << RXC0)));

    /* returneaza datele din buffer */
    return UDR0;
}

void direction (int dir)
{
	if (dir == 2) { 			/* Fata */
		PORTC &= ~(1 << PC3);
		PORTC |= (1 << PC4);
	} else if (dir == 1) {  	/* Spate */
		PORTC &= ~(1 << PC4);
		PORTC |= (1 << PC3);
	} else if (dir == 0) { 	/* Stop */
		PORTC &= ~(1 << PC3);
		PORTC &= ~(1 << PC4);
	}
	
	if (dir == 4) { 			/* Stanga */
		PORTC &= ~(1 << PC6);
		PORTC |= (1 << PC5);
	} else if (dir == 3) { 	/* Dreapta */
		PORTC &= ~(1 << PC5);
		PORTC |= (1 << PC6);
	} else if (dir == 5) { 	/* Stop */
		PORTC &= ~(1 << PC5);
		PORTC &= ~(1 << PC6);
	}
}

void PWM_speed(int speed) {
	
	/* Se utilizeaza Timer0 pe 8 biti */
	TCCR0A |= (1 << WGM01) | (1 << WGM00);
	TCCR0A |= (1 << COM0A1);
	TCCR0B |= (1 << CS00);
	
	/* Seteaza pragul OCR0A*/
	OCR0A = speed;
}

void run()
{
	char message;
	
	while (1) {
		
		message = USART_receive();	
		
		/* Seteaza directie motoare */
		if (message == 'F') {
			direction(1);
		}
		if (message == 'B') {
			direction(2);
		}
		if (message == 'L') {
			direction(3);
		}
		if (message == 'R') {
			direction(4);
		}
		if (message == 'G') {
			direction(1);
			direction(3);
		}
		if (message == 'I') {
			direction(1);
			direction(4);
		}
		if (message == 'H') {
			direction(2);
			direction(3);
		}
		if (message == 'J') {
			direction(2);
			direction(4);
		}
		if (message == 'S') {
			direction(0);
			direction(5);
		}
		
		/* Seteaza viteza */
		if (message == 'q') {
			PWM_speed (255);
		}
		if ((message > '0') && (message < '4')) {
			PWM_speed (130);
		}
		if ((message > '3') && (message < '7')) {
			PWM_speed (190);
		}
		if ((message > '6') && (message <= '9')) {
			PWM_speed (250);
		}
	}
}

int main (void) 
{
	USART_init();
	motor_init();
	
	run();
	
	return 0;
}